@@ -17,8 +17,7 @@ G_DEFINE_TYPE (EmpathyRosterView, empathy_roster_view, EGG_TYPE_LIST_BOX)
 
 enum
 {
-  PROP_MANAGER = 1,
-  PROP_MODEL,
+  PROP_MODEL = 1,
   PROP_SHOW_OFFLINE,
   PROP_SHOW_GROUPS,
   PROP_EMPTY,
@@ -40,8 +39,6 @@ static guint signals[LAST_SIGNAL];
 
 struct _EmpathyRosterViewPriv
 {
-  EmpathyIndividualManager *manager;
-
   /* FolksIndividual (borrowed) -> GHashTable (
    * (gchar * group_name) -> EmpathyRosterContact (borrowed))
    *
@@ -114,9 +111,6 @@ empathy_roster_view_get_property (GObject *object,
 
   switch (property_id)
     {
-      case PROP_MANAGER:
-        g_value_set_object (value, self->priv->manager);
-        break;
       case PROP_MODEL:
         g_value_set_object (value, self->priv->model);
         break;
@@ -145,10 +139,6 @@ empathy_roster_view_set_property (GObject *object,
 
   switch (property_id)
     {
-      case PROP_MANAGER:
-        g_assert (self->priv->manager == NULL); /* construct only */
-        self->priv->manager = g_value_dup_object (value);
-        break;
       case PROP_MODEL:
         g_assert (self->priv->model == NULL);
         self->priv->model = g_value_dup_object (value);
@@ -229,9 +219,9 @@ ensure_roster_group (EmpathyRosterView *self,
   if (roster_group != NULL)
     return EMPATHY_ROSTER_GROUP (roster_group);
 
-  if (!tp_strdiff (group, EMPATHY_ROSTER_VIEW_GROUP_TOP_GROUP))
+  if (!tp_strdiff (group, EMPATHY_ROSTER_MODEL_GROUP_TOP_GROUP))
     roster_group = empathy_roster_group_new (group, "emblem-favorite-symbolic");
-  else if (!tp_strdiff (group, EMPATHY_ROSTER_VIEW_GROUP_PEOPLE_NEARBY))
+  else if (!tp_strdiff (group, EMPATHY_ROSTER_MODEL_GROUP_PEOPLE_NEARBY))
     roster_group = empathy_roster_group_new (group, "im-local-xmpp");
   else
     roster_group = empathy_roster_group_new (group, NULL);
@@ -316,14 +306,13 @@ individual_added (EmpathyRosterView *self,
       GList *groups, *l;
       GList *tops;
 
-      tops = empathy_individual_manager_get_top_individuals (
-          self->priv->manager);
+      tops = empathy_roster_model_get_top_individuals (self->priv->model);
 
       if (folks_favourite_details_get_is_favourite (
             FOLKS_FAVOURITE_DETAILS (individual)) ||
           g_list_index (tops, individual) != -1)
         {
-          add_to_group (self, individual, EMPATHY_ROSTER_VIEW_GROUP_TOP_GROUP);
+          add_to_group (self, individual, EMPATHY_ROSTER_MODEL_GROUP_TOP_GROUP);
         }
 
       groups = empathy_roster_model_get_groups_for_individual (self->priv->model,
@@ -339,7 +328,7 @@ individual_added (EmpathyRosterView *self,
       else
         {
           /* No group, adds to Ungrouped */
-          add_to_group (self, individual, EMPATHY_ROSTER_VIEW_GROUP_UNGROUPED);
+          add_to_group (self, individual, EMPATHY_ROSTER_MODEL_GROUP_UNGROUPED);
         }
 
       g_list_free (groups);
@@ -521,37 +510,6 @@ compare_roster_contacts_by_alias (EmpathyRosterContact *a,
   return g_ascii_strcasecmp (alias_a, alias_b);
 }
 
-static gboolean
-contact_is_favourite (EmpathyRosterContact *contact)
-{
-  FolksIndividual *individual;
-
-  individual = empathy_roster_contact_get_individual (contact);
-
-  return folks_favourite_details_get_is_favourite (
-      FOLKS_FAVOURITE_DETAILS (individual));
-}
-
-static gboolean
-contact_in_top (EmpathyRosterView *self,
-    EmpathyRosterContact *contact)
-{
-  FolksIndividual *individual;
-  GList *tops;
-
-  if (contact_is_favourite (contact))
-    return TRUE;
-
-  individual = empathy_roster_contact_get_individual (contact);
-
-  tops = empathy_individual_manager_get_top_individuals (self->priv->manager);
-
-  if (g_list_index (tops, individual) != -1)
-    return TRUE;
-
-  return FALSE;
-}
-
 static gint
 compare_roster_contacts_no_group (EmpathyRosterView *self,
     EmpathyRosterContact *a,
@@ -559,8 +517,8 @@ compare_roster_contacts_no_group (EmpathyRosterView *self,
 {
   gboolean top_a, top_b;
 
-  top_a = contact_in_top (self, a);
-  top_b = contact_in_top (self, b);
+  top_a = empathy_roster_model_contact_in_top (self->priv->model, a);
+  top_b = empathy_roster_model_contact_in_top (self->priv->model, b);
 
   if (top_a == top_b)
     /* Both contacts are in the top of the roster (or not). Sort them
@@ -576,15 +534,15 @@ static gint
 compare_group_names (const gchar *group_a,
     const gchar *group_b)
 {
-  if (!tp_strdiff (group_a, EMPATHY_ROSTER_VIEW_GROUP_TOP_GROUP))
+  if (!tp_strdiff (group_a, EMPATHY_ROSTER_MODEL_GROUP_TOP_GROUP))
     return -1;
 
-  if (!tp_strdiff (group_b, EMPATHY_ROSTER_VIEW_GROUP_TOP_GROUP))
+  if (!tp_strdiff (group_b, EMPATHY_ROSTER_MODEL_GROUP_TOP_GROUP))
     return 1;
 
-  if (!tp_strdiff (group_a, EMPATHY_ROSTER_VIEW_GROUP_UNGROUPED))
+  if (!tp_strdiff (group_a, EMPATHY_ROSTER_MODEL_GROUP_UNGROUPED))
     return 1;
-  else if (!tp_strdiff (group_b, EMPATHY_ROSTER_VIEW_GROUP_UNGROUPED))
+  else if (!tp_strdiff (group_b, EMPATHY_ROSTER_MODEL_GROUP_UNGROUPED))
     return -1;
 
   return g_ascii_strcasecmp (group_a, group_b);
@@ -783,7 +741,8 @@ contact_should_be_displayed (EmpathyRosterView *self,
   if (self->priv->show_offline)
       return TRUE;
 
-  if (contact_is_favourite (contact))
+  if (empathy_roster_model_contact_in_top (self->priv->model,
+          contact))
     {
       const gchar *group_name;
 
@@ -793,7 +752,7 @@ contact_should_be_displayed (EmpathyRosterView *self,
 
       group_name = empathy_roster_contact_get_group (contact);
 
-      if (!tp_strdiff (group_name, EMPATHY_ROSTER_VIEW_GROUP_TOP_GROUP))
+      if (!tp_strdiff (group_name, EMPATHY_ROSTER_MODEL_GROUP_TOP_GROUP))
         /* Always display favourite contact in group mode only in the
          * 'top group'*/
         return TRUE;
@@ -932,7 +891,7 @@ remove_from_group (EmpathyRosterView *self,
 
   if (g_hash_table_size (contacts) == 0)
     {
-      add_to_group (self, individual, EMPATHY_ROSTER_VIEW_GROUP_UNGROUPED);
+      add_to_group (self, individual, EMPATHY_ROSTER_MODEL_GROUP_UNGROUPED);
     }
 
   roster_group = lookup_roster_group (self, group);
@@ -959,10 +918,10 @@ update_top_contacts (EmpathyRosterView *self)
       return;
     }
 
-  tops = empathy_individual_manager_get_top_individuals (self->priv->manager);
+  tops = empathy_roster_model_get_top_individuals (self->priv->model);
 
   group = g_hash_table_lookup (self->priv->roster_groups,
-      EMPATHY_ROSTER_VIEW_GROUP_TOP_GROUP);
+      EMPATHY_ROSTER_MODEL_GROUP_TOP_GROUP);
   if (group == NULL)
     {
       to_add = g_list_copy (tops);
@@ -979,7 +938,8 @@ update_top_contacts (EmpathyRosterView *self)
           EmpathyRosterContact *contact = l->data;
           FolksIndividual *individual;
 
-          if (contact_is_favourite (contact))
+          if (empathy_roster_model_contact_in_top (self->priv->model,
+                  contact))
             continue;
 
           individual = empathy_roster_contact_get_individual (contact);
@@ -999,10 +959,10 @@ update_top_contacts (EmpathyRosterView *self)
     }
 
   for (l = to_add; l != NULL; l = g_list_next (l))
-    add_to_group (self, l->data, EMPATHY_ROSTER_VIEW_GROUP_TOP_GROUP);
+    add_to_group (self, l->data, EMPATHY_ROSTER_MODEL_GROUP_TOP_GROUP);
 
   for (l = to_remove; l != NULL; l = g_list_next (l))
-    remove_from_group (self, l->data, EMPATHY_ROSTER_VIEW_GROUP_TOP_GROUP);
+    remove_from_group (self, l->data, EMPATHY_ROSTER_MODEL_GROUP_TOP_GROUP);
 
   g_list_free (to_add);
   g_list_free (to_remove);
@@ -1029,15 +989,14 @@ groups_changed_cb (EmpathyRosterModel *model,
 }
 
 static void
-top_individuals_changed_cb (EmpathyIndividualManager *manager,
-    GParamSpec *spec,
+top_individuals_changed_cb (EmpathyRosterModel *model,
     EmpathyRosterView *self)
 {
   update_top_contacts (self);
 }
 
 static void
-favourites_changed_cb (EmpathyIndividualManager *manager,
+favourites_changed_cb (EmpathyRosterModel *model,
     FolksIndividual *individual,
     gboolean favourite,
     EmpathyRosterView *self)
@@ -1051,10 +1010,10 @@ favourites_changed_cb (EmpathyIndividualManager *manager,
   if (self->priv->show_groups)
     {
       if (favourite)
-        add_to_group (self, individual, EMPATHY_ROSTER_VIEW_GROUP_TOP_GROUP);
+        add_to_group (self, individual, EMPATHY_ROSTER_MODEL_GROUP_TOP_GROUP);
       else
         remove_from_group (self, individual,
-            EMPATHY_ROSTER_VIEW_GROUP_TOP_GROUP);
+            EMPATHY_ROSTER_MODEL_GROUP_TOP_GROUP);
     }
   else
     {
@@ -1078,7 +1037,6 @@ empathy_roster_view_constructed (GObject *object)
   if (chain_up != NULL)
     chain_up (object);
 
-  g_assert (EMPATHY_IS_INDIVIDUAL_MANAGER (self->priv->manager));
   g_assert (EMPATHY_IS_ROSTER_MODEL (self->priv->model));
 
   populate_view (self);
@@ -1089,9 +1047,9 @@ empathy_roster_view_constructed (GObject *object)
       G_CALLBACK (individual_removed_cb), self, 0);
   tp_g_signal_connect_object (self->priv->model, "groups-changed",
       G_CALLBACK (groups_changed_cb), self, 0);
-  tp_g_signal_connect_object (self->priv->manager, "notify::top-individuals",
+  tp_g_signal_connect_object (self->priv->model, "top-individuals-changed",
       G_CALLBACK (top_individuals_changed_cb), self, 0);
-  tp_g_signal_connect_object (self->priv->manager, "notify::favourites-changed",
+  tp_g_signal_connect_object (self->priv->model, "favourites-changed",
       G_CALLBACK (favourites_changed_cb), self, 0);
 
   egg_list_box_set_sort_func (EGG_LIST_BOX (self),
@@ -1115,7 +1073,6 @@ empathy_roster_view_dispose (GObject *object)
   stop_flashing (self);
 
   empathy_roster_view_set_live_search (self, NULL);
-  g_clear_object (&self->priv->manager);
   g_clear_object (&self->priv->model);
 
   if (chain_up != NULL)
@@ -1343,12 +1300,6 @@ empathy_roster_view_class_init (
 
   box_class->child_activated = empathy_roster_view_child_activated;
 
-  spec = g_param_spec_object ("manager", "Manager",
-      "EmpathyIndividualManager",
-      EMPATHY_TYPE_INDIVIDUAL_MANAGER,
-      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
-  g_object_class_install_property (oclass, PROP_MANAGER, spec);
-
   spec = g_param_spec_object ("model", "Model",
       "EmpathyRosterModel",
       EMPATHY_TYPE_ROSTER_MODEL,
@@ -1422,24 +1373,15 @@ empathy_roster_view_init (EmpathyRosterView *self)
 }
 
 GtkWidget *
-empathy_roster_view_new (EmpathyIndividualManager *manager,
-    EmpathyRosterModel *model)
+empathy_roster_view_new (EmpathyRosterModel *model)
 {
-  g_return_val_if_fail (EMPATHY_IS_INDIVIDUAL_MANAGER (manager), NULL);
   g_return_val_if_fail (EMPATHY_IS_ROSTER_MODEL (model), NULL);
 
   return g_object_new (EMPATHY_TYPE_ROSTER_VIEW,
-      "manager", manager,
       "model", model,
       NULL);
 }
 
-EmpathyIndividualManager *
-empathy_roster_view_get_manager (EmpathyRosterView *self)
-{
-  return self->priv->manager;
-}
-
 void
 empathy_roster_view_show_offline (EmpathyRosterView *self,
     gboolean show)
