@@ -29,6 +29,8 @@
 
 #include <libempathy/empathy-gsettings.h>
 
+#include <libempathy-gtk/empathy-theme-manager.h>
+
 #define DEBUG_FLAG EMPATHY_DEBUG_OTHER
 #include <libempathy/empathy-debug.h>
 
@@ -38,7 +40,7 @@
  * If the number stored in gsettings is lower than it, all the tasks will
  * be executed.
  */
-#define SANITY_CLEANING_NUMBER 2
+#define SANITY_CLEANING_NUMBER 3
 
 static void
 account_update_parameters_cb (GObject *source,
@@ -73,7 +75,7 @@ fix_xmpp_account_priority (TpAccountManager *am)
       GHashTable *params;
       gint priority;
 
-      if (tp_strdiff (tp_account_get_protocol (account), "jabber"))
+      if (tp_strdiff (tp_account_get_protocol_name (account), "jabber"))
         continue;
 
       params = (GHashTable *) tp_account_get_parameters (account);
@@ -140,6 +142,62 @@ set_facebook_account_fallback_server (TpAccountManager *am)
   g_list_free (accounts);
 }
 
+static void
+upgrade_chat_theme_settings (void)
+{
+  GSettings *gsettings_chat;
+  gchar *theme, *new_theme = NULL;
+  const char *variant = "";
+
+  gsettings_chat = g_settings_new (EMPATHY_PREFS_CHAT_SCHEMA);
+
+  theme = g_settings_get_string (gsettings_chat,
+      EMPATHY_PREFS_CHAT_THEME);
+
+  if (!tp_strdiff (theme, "adium")) {
+    gchar *path;
+
+    path = g_settings_get_string (gsettings_chat,
+        EMPATHY_PREFS_CHAT_ADIUM_PATH);
+
+    new_theme = empathy_theme_manager_dup_theme_name_from_path (path);
+    if (new_theme == NULL)
+      {
+        /* Use the Classic theme as fallback */
+        new_theme = g_strdup ("Classic");
+      }
+
+    g_free (path);
+  } else if (!tp_strdiff (theme, "gnome")) {
+    new_theme = g_strdup ("PlanetGNOME");
+  } else if (!tp_strdiff (theme, "simple")) {
+    new_theme = g_strdup ("Boxes");
+    variant = "Simple";
+  } else if (!tp_strdiff (theme, "clean")) {
+    new_theme = g_strdup ("Boxes");
+    variant = "Clean";
+  } else if (!tp_strdiff (theme, "blue")) {
+    new_theme = g_strdup ("Boxes");
+    variant = "Blue";
+  } else {
+    /* Assume that's an Adium theme name. The theme manager will fallback to
+     * 'Classic' if it can't find it. */
+    goto finally;
+  }
+
+  DEBUG ("Migrating to '%s' variant '%s'", new_theme, variant);
+
+  g_settings_set_string (gsettings_chat,
+    EMPATHY_PREFS_CHAT_THEME, new_theme);
+  g_settings_set_string (gsettings_chat,
+    EMPATHY_PREFS_CHAT_THEME_VARIANT, variant);
+
+finally:
+  g_free (theme);
+  g_free (new_theme);
+  g_object_unref (gsettings_chat);
+}
+
 static void
 run_sanity_cleaning_tasks (TpAccountManager *am)
 {
@@ -147,6 +205,7 @@ run_sanity_cleaning_tasks (TpAccountManager *am)
 
   fix_xmpp_account_priority (am);
   set_facebook_account_fallback_server (am);
+  upgrade_chat_theme_settings ();
 }
 
 static void
